package cn.timer.api.controller.jxgl;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.conditions.query.LambdaQueryChainWrapper;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;

import cn.hutool.core.bean.BeanUtil;
import cn.timer.api.bean.jxgl.JxglBasicSetting;
import cn.timer.api.bean.jxgl.JxglPerformanceRating;
import cn.timer.api.bean.qyzx.QyzxEmpEntAsso;
import cn.timer.api.config.annotation.CurrentUser;
import cn.timer.api.config.annotation.UserBean;
import cn.timer.api.config.enums.CommonEnum;
import cn.timer.api.dao.jxgl.JxglBasicSettingMapper;
import cn.timer.api.dao.jxgl.JxglPerformanceRatingMapper;
import cn.timer.api.dao.qyzx.QyzxEmpEntAssoMapper;
import cn.timer.api.dao.yggl.YgglMainEmpMapper;
import cn.timer.api.dto.jxgl.BasicSettingDto;
import cn.timer.api.utils.Result;
import cn.timer.api.utils.ResultUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

@Api(tags = "3.1绩效管理")
@RestController
@Transactional(rollbackFor = Exception.class)
@RequestMapping(value = "/jxgl",  
produces = { "application/json","multipart/form-data" })
public class JxglController {

	@Autowired
	private QyzxEmpEntAssoMapper qyzxEmpEntAssoMapper;
	
	@Autowired
	private JxglBasicSettingMapper jxglBasicSettingMapper;
	
	@Autowired
	private JxglPerformanceRatingMapper jxglPerformanceRatingMapper;
	
	//TODO 基础设置
	/**
	 * 新增或编辑-基础设置
	 */
	@PostMapping(value = "/save_basic_setting")
	@ApiOperation(value = "1.新增或编辑-基础设置", httpMethod = "POST", notes = "新增或编辑-审批模板组")
	@ApiOperationSupport(order = 1)
	public Result<Object> saveAtg(@CurrentUser UserBean userBean ,@Validated @RequestBody BasicSettingDto basicSettingDto) throws Exception{
		Integer orgCode = userBean.getOrgCode();
		Integer count = new LambdaQueryChainWrapper<QyzxEmpEntAsso>(qyzxEmpEntAssoMapper)
				.eq(QyzxEmpEntAsso::getEmpNum, userBean.getEmpNum())
				.in(QyzxEmpEntAsso::getUserType, CommonEnum.U_TYPE_ADMIN.getType(),CommonEnum.U_TYPE_C_ADMIN.getType())
				.count();
		
		if (count <= 0) {
			return ResultUtil.error("无权限编辑");
		}
		
		List<JxglPerformanceRating> listPr = basicSettingDto.getPerformanceRatings();
		if (listPr != null) {
			for (JxglPerformanceRating pr : listPr) {
				if (pr.getSectionMaxScore() > basicSettingDto.getMaxScore()) {
					return ResultUtil.error("区间值不能超过 最高分");
				}else if (pr.getSectionMinScore() >= pr.getSectionMaxScore()) {
					return ResultUtil.error("区间值 设置不正确");
				}
			}
		}
		JxglBasicSetting bS = jxglBasicSettingMapper.selectOne(new QueryWrapper<JxglBasicSetting>().lambda()
				.select(JxglBasicSetting::getId)
				.eq(JxglBasicSetting::getOrgCode, orgCode));
		if (bS != null) {
			// 删除 绩效等级
			jxglPerformanceRatingMapper.delete(new QueryWrapper<JxglPerformanceRating>().lambda()
					.eq(JxglPerformanceRating::getBasicSettingId, bS.getId()));
			// 删除 基础设置
			jxglBasicSettingMapper.delete(new QueryWrapper<JxglBasicSetting>().lambda()
					.eq(JxglBasicSetting::getOrgCode, orgCode));
		}
		
		
		JxglBasicSetting bS2 = JxglBasicSetting.builder().build();
		BeanUtil.copyProperties(basicSettingDto, bS2, "performanceRatings");
		bS2.setOrgCode(orgCode);
		bS2.insert();
		if (listPr != null) {
			for (JxglPerformanceRating pr : listPr) {
				pr.setBasicSettingId(bS2.getId());
				jxglPerformanceRatingMapper.insert(pr);
			}
		}

		return ResultUtil.success();

	}
	
}
